// action.cpp

#include "action.h"

const Action::Pattern Action::_patterns[] = {
    Pattern("<delete>", "delete action"),
    Pattern("Execute",
            "run an executable",
            "execute", Value::STRING, 0,
            "specifies the executable to be run along with any arguments "\
            "to pass it, the tilde (~) character will be expanded in "\
            "this string to your home directory, but no other shell "\
            "expansions or scripring syntax can be used in the command"),
    Pattern("DirectionalFocusNorth",
            "cycles focus to the window north of the currently focused "\
            "window",
            "dialog", Value::BOOLEAN, 1,
            "display the focus cycling dialog while switching focus"),
    Pattern("DirectionalFocusEast",
            "cycles focus to the window east of the currently focused "\
            "window",
            "dialog", Value::BOOLEAN, 1,
            "display the focus cycling dialog while switching focus"),
    Pattern("DirectionalFocusSouth",
            "cycles focus to the window south of the currently focused "\
            "window",
            "dialog", Value::BOOLEAN, 1,
            "display the focus cycling dialog while switching focus"),
    Pattern("DirectionalFocusWest",
            "cycles focus to the window west of the currently focused "\
            "window",
            "dialog", Value::BOOLEAN, 1,
            "display the focus cycling dialog while switching focus"),
    Pattern("DirectionalFocusNorthEast",
            "cycles focus to the window northeast of the currently "\
            "focused window",
            "dialog", Value::BOOLEAN, 1,
            "display the focus cycling dialog while switching focus"),
    Pattern("DirectionalFocusSouthEast",
            "cycles focus to the window southeast of the currently "\
            "focused window",
            "dialog", Value::BOOLEAN, 1,
            "display the focus cycling dialog while switching focus"),
    Pattern("DirectionalFocusSouthWest",
            "cycles focus to the window southwest of the currently "\
            "focused window",
            "dialog", Value::BOOLEAN, 1,
            "display the focus cycling dialog while switching focus"),
    Pattern("DirectionalFocusNorthWest",
            "cycles focus to the window northwest of the currently "\
            "focused window",
            "dialog", Value::BOOLEAN, 1,
            "display the focus cycling dialog while switching focus"),
    Pattern("NextWindow",
            "cycles focus to the next window",
            "linear", Value::BOOLEAN, 0,
            "when true, windows are always cycled in the order in which "\
            "they appeared; otherwise, they are cycled in order of recent "\
            "use",
            "dialog", Value::BOOLEAN, 1,
            "display the focus cycling dialog while switching focus"),
    Pattern("PreviousWindow",
            "cycles focus to the previous window",
            "linear", Value::BOOLEAN, 0,
            "when true, windows are always cycled in the order in which "\
            "they appeared; otherwise, they are cycled in order of recent "\
            "use",
            "dialog", Value::BOOLEAN, 1,
            "display the focus cycling dialog while switching focus"),
    Pattern("Desktop",
            "changes the currently visible desktop",
            "desktop", Value::NUMBER, 1,
            "the number of the desktop (starting at 1) to make visible",
            "dialog", Value::BOOLEAN, 0,
            "display the desktop dialog (with pager) while switching "\
            "desktops, this will only work in keyboard bindings, "\
            "otherwise, it is disabled"),
    Pattern("DesktopNext",
            "makes the next desktop visible",
            "wrap", Value::BOOLEAN, 1,
            "whether to wrap around to the first desktop when going past "\
            "the last",
            "dialog", Value::BOOLEAN, 1,
            "display the desktop dialog (with pager) while switching "\
            "desktops, this will only work in keyboard bindings, "\
            "otherwise, it is disabled"),
    Pattern("DesktopPrevious",
            "makes the previous desktop visible",
            "wrap", Value::BOOLEAN, 1,
            "whether to wrap around to the last desktop when going past "\
            "the first",
            "dialog", Value::BOOLEAN, 1,
            "display the desktop dialog (with pager) while switching "\
            "desktops, this will only work in keyboard bindings, "\
            "otherwise, it is disabled"),
    Pattern("DesktopRight",
            "makes visible the desktop to the right of the currrently "\
            "visible desktop",
            "wrap", Value::BOOLEAN, 1,
            "whether to wrap around to the first desktop when going past "\
            "the last",
            "dialog", Value::BOOLEAN, 1,
            "display the desktop dialog (with pager) while switching "\
            "desktops, this will only work in keyboard bindings, "\
            "otherwise, it is disabled"),
    Pattern("DesktopLeft",
            "makes visible the desktop to the left of the currently "\
            "visible desktop",
            "wrap", Value::BOOLEAN, 1,
            "whether to wrap around to the last desktop when going past "\
            "the first",
            "dialog", Value::BOOLEAN, 1,
            "display the desktop dialog (with pager) while switching "\
            "desktops, this will only work in keyboard bindings, "\
            "otherwise, it is disabled"),
    Pattern("DesktopDown",
            "makes visible the desktop beneath the currently visible "\
            "desktop",
            "wrap", Value::BOOLEAN, 1,
            "whether to wrap around to the first desktop when going past "\
            "the last",
            "dialog", Value::BOOLEAN, 1,
            "display the desktop dialog (with pager) while switching "\
            "desktops, this will only work in keyboard bindings, "\
            "otherwise, it is disabled"),
    Pattern("DesktopUp",
            "makes visible the desktop above the currently visible desktop",
            "wrap", Value::BOOLEAN, 1,
            "whether to wrap around to the last desktop when going past "\
            "the first",
            "dialog", Value::BOOLEAN, 1,
            "display the desktop dialog (with pager) while switching "\
            "desktops, this will only work in keyboard bindings, "\
            "otherwise, it is disabled"),
    Pattern("ShowDesktop",
            "hides all windows so that the desktop is visible; if you are "\
            "running a desktop manager such as nautilus, then the desktop "\
            "window will also be focused"),
    Pattern("UnshowDesktop",
            "shows windows that were hidden by ShowDesktop and restores "\
            "focus"),
    Pattern("ToggleShowDesktop",
            "toggles between ShowDesktop and UnshowDesktop"),
    Pattern("DesktopLast",
            "makes the most recently visible desktop visible again, only "\
            "one desktop is remembered, so using this twice will simply "\
            "flip between two desktops"),
    Pattern("ShowMenu",
            "shows a menu by name",
            "menu", Value::STRING, 0,
            "the name of the menu to show, the names of menus are "\
            "specified in the id attribute of their menu tag"),
    Pattern("Reconfigure",
            "reloads the config file, menu, and theme"),
    Pattern("Restart",
            "restarts Openbox, this starts a new copy of Openbox, and can "\
            "be used to upgrade to a newly installed version",
            "execute", Value::STRING, 0,
            "when this is used, the executable specified within will be "\
            "run instead of Openbox, this can be used to switch to "\
            "another window manager if desired"),
    Pattern("Exit",
            "exits Openbox"),
    Pattern("Activate",
            "activates the window, this is the same process that occurs "\
            "when focus cycling to a window, or selecting it in the "\
            "client-list-menu"),
    Pattern("Focus",
            "give focus to the window"),
    Pattern("Unfocus",
            "remove focus from the window"),
    Pattern("Iconify",
            "iconify the window, this causes the window to not be shown "\
            "on screen, the window can be found again with focus cycling "\
            "or with the client-list-menu"),
    Pattern("RaiseLower",
            "raise the window to the forefront if it is obscured behind "\
            "anything, lower it to the back if it is already at the front"),
    Pattern("Raise",
            "raise the window to the forefront"),
    Pattern("Lower",
            "lower the window to the back"),
    Pattern("Close",
            "request the window to close"),
    Pattern("Kill",
            "forcibly disconnect the window's application from the X "\
            "server"),
    Pattern("ShadeLower",
            "shade the window, if the window is already shaded, then "\
            "Lower it"),
    Pattern("UnshadeRaise",
            "unshade the window, if the window is already unshaded, then "\
            "raise it"),
    Pattern("Shade",
            "shade the window, this hides everything of the window except "\
            "the titlebar, this concept has also been known as \"rolling "\
            "up\" the window into the titlebar"),
    Pattern("Unshade",
            "unshade the window, this shows the entire window if it was "\
            "previously shaded"),
    Pattern("ToggleShade",
            "toggles the shaded state of the window, firing the Shade and "\
            "Unshade actions appropriately"),
    Pattern("ToggleOmnipresent",
            "toggles between moving the client to all desktops "\
            "(omnipresent) and to the currently visible desktop"),
    Pattern("MoveRelativeHorz",
            "moves the window horizontally",
            "delta", Value::NUMBER, 0,
            "the number of pixels to move the window, a positive value "\
            "moves to the right, a negative value moves to the left"),
    Pattern("MoveRelativeVert",
            "move the window vertically",
            "delta", Value::NUMBER, 0,
            "the number of pixels to move the window, a positive value "\
            "moves down, a negative value moves up"),
    Pattern("ResizeRelativeHorz",
            "resizes the window horizontally",
            "delta", Value::NUMBER, 0,
            "the amount to resize the window, a positive value makes the "\
            "window wider, a negative value makes it skinnier"),
    Pattern("ResizeRelativeVert",
            "resizes the window vertically",
            "delta", Value::NUMBER, 0,
            "the amount to resize the window, a positive value makes the "\
            "window taller, a negative value makes it shorter"),
    Pattern("MaximizeFull",
            "maximizes a window to fill the entire screen"),
    Pattern("UnmaximizeFull",
            "restores a window to its original size"),
    Pattern("ToggleMaximizeFull",
            "toggles between MaximizeFull and UnmaximizeFull"),
    Pattern("MaximizeHorz",
            "maximizes a window to fill the entire screen horizontally, "\
            "but not vertically"),
    Pattern("UnmaximizeHorz",
            "restores a window to its original size"),
    Pattern("ToggleMaximizeHorz",
            "toggles between MaximizeHorz and UnmaximizeHorz"),
    Pattern("MaximizeVert",
            "maximizes a window to fill the entire screen vertically, but "\
            "not horizontally"),
    Pattern("UnmaximizeVert",
            "restores a window to its original size"),
    Pattern("ToggleMaximizeVert",
            "toggles between MaximizeHorz and UnmaximizeVert"),
    Pattern("SendToDesktop",
            "sends the window to the specified desktop",
            "desktop", Value::NUMBER, 1,
            "the number of the desktop (starting at 1) to send the "\
            "window to",
            "follow", Value::BOOLEAN, 1,
            "whether to change to the desktop while moving the window"),
    Pattern("SendToDesktopNext",
            "sends the window to the next desktop",
            "wrap", Value::BOOLEAN, 1,
            "whether to wrap around to the first desktop when going past "\
            "the last",
            "follow", Value::BOOLEAN, 1,
            "whether to change to the desktop while moving the window"),
    Pattern("SendToDesktopPrevious",
            "sends the window to the previous desktop",
            "wrap", Value::BOOLEAN, 1,
            "whether to wrap around to the last desktop when going past "\
            "the first",
            "follow", Value::BOOLEAN, 1,
            "whether to change to the desktop while moving the window"),
    Pattern("SendToDesktopRight",
            "sends the window to the desktop to the right of the "\
            "currently visible desktop",
            "wrap", Value::BOOLEAN, 1,
            "whether to wrap around to the first desktop when going past "\
            "the last",
            "follow", Value::BOOLEAN, 1,
            "whether to change to the desktop while moving the window"),
    Pattern("SendToDesktopLeft",
            "sends the window to the desktop to the left of the currently "\
            "visible desktop",
            "wrap", Value::BOOLEAN, 1,
            "whether to wrap around to the last desktop when going past "\
            "the first",
            "follow", Value::BOOLEAN, 1,
            "whether to change to the desktop while moving the window"),
    Pattern("SendToDesktopDown",
            "sends the window to the desktop beneath the currently "\
            "visible desktop",
            "wrap", Value::BOOLEAN, 1,
            "whether to wrap around to the first desktop when going past "\
            "the last",
            "follow", Value::BOOLEAN, 1,
            "whether to change to the desktop while moving the window"),
    Pattern("SendToDesktopUp",
            "sends the window to the desktop above the currently visible "\
            "desktops",
            "wrap", Value::BOOLEAN, 1,
            "whether to wrap around to the last desktop when going past "\
            "the first",
            "follow", Value::BOOLEAN, 1,
            "whether to change to the desktop while moving the window"),
    Pattern("ToggleDecorations",
            "toggles if the window is given decorations, decorations are "\
            "the the titlebar and handle that are shown around most "\
            "windows"),
    Pattern("Move",
            "begins an interactive move, the move will complete when the "\
            "mouse button is released, if it is bound to one, the Enter "\
            "key will also complete the move, and the Escape key will "\
            "cancel it"),
    Pattern("Resize",
            "begins an interactive resize, the resize will complete when "\
            "the mouse button is released, if it is bound to one, the "\
            "Enter key will also complete the resize, and the Escape key "\
            "will cancel it"),
    Pattern("SendToTopLayer",
            "makes the window always-on-top"),
    Pattern("SendToNormalLayer",
            "make the window not always-on-top or always-on-bottom"),
    Pattern("SendToBottomLayer",
            "makes the window always-on-bottom"),
    Pattern("ToggleAllwaysOnTop",
            "toggles between SendToTopLayer and SendToNormalLayer"),
    Pattern("ToggleAllwaysOnBottom",
            "toggles between SendToBottomLayer and SendToNormalLayer"),
    Pattern("MoveToEdgeNorth",
            "moves the window to the nearest edge to the north of it"),
    Pattern("MoveToEdgeSouth",
            "moves the window to the nearest edge to the south of it"),
    Pattern("MoveToEdgeEast",
            "moves the window to the nearest edge to the east of it"),
    Pattern("MoveToEdgeWest",
            "moves the window to the nearest edge to the west of it"),
    Pattern("GrowToEdgeNorth",
            "resizes the window up to the nearest edge to the north of it"),
    Pattern("GrowToEdgeSouth",
            "resizes the window up to the nearest edge to the south of it"),
    Pattern("GrowToEdgeEast",
            "resizes the window up to the nearest edge to the east of it"),
    Pattern("GrowToEdgeWest",
            "resizes the window up to the nearest edge to the west of it"),
};

int Action::patterns_number(const gchar *name)
{
    for(int i = 1; patterns_name(i); ++i)
        if(!utf8_cmp(name, patterns_name(i)))
            return i;
    return 0;
}

const gchar *Action::patterns_name(int i)
{
    return ( (i && i < 78) ? _patterns[i]._name : 0 );
}

const gchar *Action::patterns_note(int i)
{
    return ( (i && i < 78) ? _patterns[i]._note : 0 );
}
    
const gchar *Action::patterns_param_name(int i, int p)
{
    return (p ? _patterns[i]._nameB : _patterns[i]._nameA);
}

const gchar *Action::patterns_param_note(int i, int p)
{
    return (p ? _patterns[i]._noteB : _patterns[i]._noteA);
}
        
Value::Type Action::patterns_param_type(int i, int p)
{
    return (p ? _patterns[i]._typeB : _patterns[i]._typeA);
}

int Action::patterns_param_def( int i, int p)
{
    return (p ? _patterns[i]._defB : _patterns[i]._defA);
}

Action::Action(const xmlNodePtr node) :
    _node(node),
    _pattern(patterns_number(xml_attr(node, "name")))
{

    // make action sub values
    for(int i = 0; i < 2; ++i)
        if(patterns_param_name(_pattern, i))
            switch(patterns_param_type(_pattern, i)) {
                case Value::NUMBER:
                    _params[i] = new Value(_node,
                                           patterns_param_def(_pattern, i),
                                           -100, 100,
                                           patterns_param_name(_pattern, i));
                    break;
                case Value::BOOLEAN:
                    _params[i] = new Value(_node,
                                           patterns_param_def(_pattern, i),
                                           "yes", "no",
                                           patterns_param_name(_pattern, i));
                    break;
                default: // Value::STRING
                    _params[i] = new Value(_node, 0,
                                           patterns_param_name(_pattern, i));
            }
        else
            _params[i] = 0;
}

void Action::set_pattern(int i)
{
    if(patterns_name(i)) {
        _pattern = i;
        xml_attr(_node, "name", patterns_name(_pattern));
        for(int i = 0; i < 2; ++i) {
            if(_params[i]) {
                _params[i]->remove();
                delete _params[i];
                _params[i] = 0;
            }
            if(patterns_param_name(_pattern, i))
                switch(patterns_param_type(_pattern, i)) {
                    case Value::NUMBER:
                        _params[i] = new Value(_node,
                                               patterns_param_def(_pattern, i),
                                               -100, 100,
                                               patterns_param_name(_pattern, i));
                        break;
                    case Value::BOOLEAN:
                        _params[i] = new Value(_node,
                                               patterns_param_def(_pattern, i),
                                               "yes", "no",
                                               patterns_param_name(_pattern, i));
                        break;
                    default: // Value::STRING
                        _params[i] = new Value(_node, 0,
                                               patterns_param_name(_pattern, i));
                }
        }
    }
}

int Action::get_pattern() const
{
    return _pattern; 
}

const gchar *Action::name() const
{
    return patterns_name(_pattern);
}

const gchar *Action::note() const
{
    return patterns_note(_pattern);
}

const gchar *Action::param_note(int i)
{
    return patterns_param_note(_pattern, i);
}

Value *Action::param(int i) const
{
    return _params[i];
}

void Action::remove()
{
    xml_remove(_node);
    _node = 0;
}

Action *Action::find_next(bool create) const
{
    xmlNodePtr n = xml_next(_node, create);
    return (n ? new Action(n) : 0);
}

