// action.h

#ifndef _action_h_included_
#define _action_h_included_

#include "value.h"

class Action
{
    // action templates
    struct Pattern {
        const gchar      *_name;
        const gchar      *_note;
        const gchar      *_nameA;
        const gchar      *_noteA;
        const Value::Type _typeA;
        const int         _defA;
        const gchar      *_nameB;
        const gchar      *_noteB;
        const Value::Type _typeB;
        const int         _defB;
        Pattern(const gchar *name,
                const gchar *note = 0,
                const gchar *nameA = 0,
                Value::Type typeA = Value::STRING,
                int defA = false,
                const gchar *noteA = 0,
                const gchar *nameB = 0,
                Value::Type typeB = Value::STRING, 
                int defB = false,
                const gchar *noteB = 0) :
            _name(name), _note(note),
            _nameA(nameA), _typeA(typeA), _defA(defA), _noteA(noteA),
            _nameB(nameB), _typeB(typeB), _defB(defB), _noteB(noteB) {}
    };
    static const Pattern _patterns[78];
    
    xmlNodePtr _node;       // xml node
    int        _pattern;    // action pattern number
    Value     *_params[2];  // action parameters

public:

    // find action pattern number by action name
    static int patterns_number(const gchar *name);
    
    // find action name by action pattern number
    static const gchar *patterns_name(int i);

    // find action note by action pattern number
    static const gchar *patterns_note(int i);
    
    // action pattern parameter name, note, type and default value
    static const gchar *patterns_param_name(int i, int p);
    static const gchar *patterns_param_note(int i, int p);
    static Value::Type patterns_param_type(int i, int p);
    static int patterns_param_def( int i, int p);

    // create action using existing node [and set pattern number]
    Action(const xmlNodePtr node);

    // get pattern number and set action by pattern number
    void set_pattern(int i);
    int get_pattern() const;
    
    // get action name, action or param description
    const gchar *name() const;
    const gchar *note() const;
    const gchar *param_note(int i);
    
    // get parameter value
    Value *param(int i) const;
    
    // remove xml node
    void remove();

    // find or create next by xml Action
    Action *find_next(bool create = false) const;

};

#endif
