// bind.h

#ifndef _bind_h_included_
#define _bind_h_included_

#include "action.h"

class Bind
{
public:
    // bind types
    enum { KEYBOARD = 0, MOUSE };

private:
    // bind types
    struct Type {
        const gchar *_name;
        const gchar *_note;
        bool         _use_action;
        const gchar *_key_name;
        Type(const gchar *name, const gchar *note,
             bool use_action, const gchar *key_name) :
            _name(name), _note(note), _use_action(use_action),
            _key_name(key_name) {}
    };
    static const Type _types[2];

    // key actions
    static const gchar *_key_action_names[];

    int            _type;      // bind type
    mutable GList *_actions;   // sub actions
    mutable GList *_binds;     // sub binds
    xmlNodePtr     _node;      // xml node pointer

public:

    // find bind by root node, path and attribute
    Bind(xmlNodePtr   node,
         const gchar *path,
         const gchar *attr_name,
         const gchar *attr_value,
         int          type);
    // create bind using existing node
    Bind(xmlNodePtr xml_node,
         int        type);
    ~Bind();

    // remove from xml
    void remove();

    // get first or next action and bind
    Action* first_action() const;
    Action* next_action() const;
    Bind* first_bind() const;
    Bind* next_bind() const;
    
    // add new action using template number, and
    // add new bind using key value and key action number
    Action *add(int i);
    Bind *add(const gchar *key, int t = 0);
    
    // delete action or bind
    void del(Action *to_delete);
    void del(Bind *to_delete);
    
    // get or set key value
    const gchar *get_key() const;
    void set_key(const gchar *key);
    
    // get or set key action number
    int get_key_action() const;
    void set_key_action(int i);

    // get bind type
    int type() const;

    // get key action name
    static const gchar *key_actions_name(int i);

    // bind type descriptions
    static const gchar *bind_types_note(int i);
};

#endif

