// fbehav.cpp

#include "fbehav.h"
#include "value.h"

namespace Behavior {
    gboolean on_change_number(GtkWidget *widget,
                              gpointer   value);
    gboolean on_change_boolean(GtkWidget *widget,
                               gpointer   value);
    gboolean on_change_string(GtkWidget   *widget,
                              GdkEventKey *event,
                              gpointer     value);
    gboolean on_change_combo(GtkComboBox *widget,
                             gpointer     value);
};

gboolean Behavior::on_change_number(GtkWidget *widget,
                                    gpointer   value)
{
    ((Value *) value)->
        set_number(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget)));
    return TRUE;
}

gboolean Behavior::on_change_boolean(GtkWidget *widget,
                                     gpointer   value)
{
    ((Value *) value)->
        set_boolean(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)));
    return TRUE;
}

gboolean Behavior::on_change_string(GtkWidget   *widget,
                                    GdkEventKey *event,
                                    gpointer     value)
{
    ((Value *) value)->
        set_string(gtk_entry_get_text(GTK_ENTRY(widget)));
    return TRUE;
}

gboolean Behavior::on_change_combo(GtkComboBox *widget,
                                   gpointer     value)
{
    gchar *str = gtk_combo_box_get_active_text(widget);
    ((Value *) value)->set_string(str);
    g_free(str);
    return TRUE;
}

GtkWidget *Behavior::face(Rc &rc)
{
    // whole page table
    GtkWidget *res = gtk_vbox_new(FALSE, 0);
    GtkWidget *table = gtk_table_new(19, 6, FALSE);
    gtk_box_pack_start(GTK_BOX(res), table, FALSE, FALSE, 0);

    // window resistance, placement, draw contents on resizing
    GtkWidget *abox = gtk_hbox_new(FALSE, 0);
    GtkWidget *label = gtk_label_new("Window placement");
    gtk_box_pack_start(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 0, 6, 0, 1);
    
    // window resistance to other window
    label = gtk_label_new("    ");
    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
    label = gtk_label_new("  ");
    gtk_table_attach_defaults(GTK_TABLE(table), label, 3, 4, 1, 2);
    static Value res_win(rc.root(), 0, 0, 100,
                         "resistance/strength");
    label = gtk_label_new("resistance to window (px):");
    abox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 1, 2, 1, 2);
    GtkAdjustment *adj = (GtkAdjustment*) gtk_adjustment_new(
            res_win.get_number(), res_win.min(), res_win.max(), 1, 5, 0);
    GtkWidget *item = gtk_spin_button_new(adj, 0, 0);
    g_signal_connect(item, "value-changed",
                     G_CALLBACK(on_change_number), &res_win);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 2, 3, 1, 2);
    
    // window resistance to screen edges
    static Value res_scr(rc.root(), 0, 0, 100,
                         "resistance/screen_edge_strength");
    label = gtk_label_new("resistance to screen (px):");
    abox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 1, 2, 2, 3);
    adj = (GtkAdjustment*) gtk_adjustment_new(
            res_scr.get_number(), res_scr.min(), res_scr.max(), 1, 5, 0);
    item = gtk_spin_button_new(adj, 0, 0);
    g_signal_connect(item, "value-changed",
                     G_CALLBACK(on_change_number), &res_scr);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 2, 3, 2, 3);

    // new window placement
    static Value win_plc(rc.root(), false, "UnderMouse", "Smart",
                         "placement/policy");
    item = gtk_check_button_new_with_label("place new window under mouse");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item),
                                 win_plc.get_boolean());
    g_signal_connect(G_OBJECT(item), "toggled",
                     G_CALLBACK(on_change_boolean), &win_plc);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 4, 6, 1, 2);

    // draw contents on resizing
    static Value win_drw(rc.root(), true, "yes", "no",
                         "resize/drawContents");
    item = gtk_check_button_new_with_label("draw contents on resizing");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item),
                                 win_drw.get_boolean());
    g_signal_connect(G_OBJECT(item), "toggled",
                     G_CALLBACK(on_change_boolean), &win_drw);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 4, 6, 2, 3);

    // separator
    GtkWidget *sep = gtk_hseparator_new();
    gtk_table_attach(GTK_TABLE(table), sep, 0, 6, 3, 4,
                     GTK_FILL, GTK_FILL, 0, 10);
    
    // window focus
    abox = gtk_hbox_new(FALSE, 0);
    label = gtk_label_new("Window focusing");
    gtk_box_pack_start(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 0, 6, 4, 5);

    // focus on create new
    static Value foc_new(rc.root(), true, "yes", "no", "focus/focusNew");
    item = gtk_check_button_new_with_label("focus new window");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item),
                                 foc_new.get_boolean());
    g_signal_connect(G_OBJECT(item), "toggled",
                     G_CALLBACK(on_change_boolean), &foc_new);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 1, 3, 5, 6);

    // focus follow mouse
    static Value foc_fol(rc.root(), false, "yes", "no", "focus/followMouse");
    item = gtk_check_button_new_with_label("focus follow mouse");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item),
                                 foc_fol.get_boolean());
    g_signal_connect(G_OBJECT(item), "toggled",
                     G_CALLBACK(on_change_boolean), &foc_fol);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 1, 3, 6, 7);

    // focus delay
    static Value foc_del(rc.root(), 0, 0, 1000, "focus/focusDelay");
    label = gtk_label_new("delay before focus (ms):");
    abox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 4, 5, 5, 6);
    adj = (GtkAdjustment*) gtk_adjustment_new(
            foc_del.get_number(), foc_del.min(), foc_del.max(), 1, 5, 0);
    item = gtk_spin_button_new(adj, 0, 0);
    g_signal_connect(item, "value-changed",
                     G_CALLBACK(on_change_number), &foc_del);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 5, 6, 5, 6);
    
    // raise on focus
    static Value foc_rai(rc.root(), false, "yes", "no", "focus/raiseOnFocus");
    item = gtk_check_button_new_with_label("raise window in focus");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item),
                                 foc_rai.get_boolean());
    g_signal_connect(G_OBJECT(item), "toggled",
                     G_CALLBACK(on_change_boolean), &foc_rai);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 4, 6, 6, 7);

    // separator
    sep = gtk_hseparator_new();
    gtk_table_attach(GTK_TABLE(table), sep, 0, 6, 7, 8,
                     GTK_FILL, GTK_FILL, 0, 10);
    
    // window focus
    abox = gtk_hbox_new(FALSE, 0);
    label = gtk_label_new("Mouse click and drag");
    gtk_box_pack_start(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 0, 6, 8, 9);

    // mouse drug threshold
    static Value mou_drg(rc.root(), 3, 0, 10, "mouse/dragThreshold");
    label = gtk_label_new("drag threshold (px):");
    abox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 1, 2, 9, 10);
    adj = (GtkAdjustment*) gtk_adjustment_new(
            mou_drg.get_number(), mou_drg.min(), mou_drg.max(), 1, 5, 0);
    item = gtk_spin_button_new(adj, 0, 0);
    g_signal_connect(item, "value-changed",
                     G_CALLBACK(on_change_number), &mou_drg);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 2, 3, 9, 10);

    // mouse double click time
    static Value mou_clk(rc.root(), 200, 0, 1000, "mouse/doubleClickTime");
    label = gtk_label_new("double click time (ms):");
    abox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 4, 5, 9, 10);
    adj = (GtkAdjustment*) gtk_adjustment_new(
            mou_clk.get_number(), mou_clk.min(), mou_clk.max(), 1, 5, 0);
    item = gtk_spin_button_new(adj, 0, 0);
    g_signal_connect(item, "value-changed",
                     G_CALLBACK(on_change_number), &mou_clk);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 5, 6, 9, 10);
    
    // separator
    sep = gtk_hseparator_new();
    gtk_table_attach(GTK_TABLE(table), sep, 0, 6, 10, 11,
                     GTK_FILL, GTK_FILL, 0, 10);
    
    // keyboard binding chain
    abox = gtk_hbox_new(FALSE, 0);
    label = gtk_label_new("Keyboard binding chain");
    gtk_box_pack_start(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 0, 6, 11, 12);

    // mouse drug threshold
    static Value key_chn(rc.root(), "C-g", "keyboard/chainQuitKey");
    label = gtk_label_new("chain quit key:");
    abox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 1, 2, 12, 13);
    item = gtk_entry_new_with_max_length(10);
    gtk_entry_set_width_chars(GTK_ENTRY(item), 10);
    gtk_entry_set_alignment(GTK_ENTRY(item), 1);
    gtk_entry_set_text(GTK_ENTRY(item), key_chn.get_string());
    g_signal_connect(G_OBJECT(item), "key-release-event",
                     G_CALLBACK(on_change_string), &key_chn);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 2, 3, 12, 13);

    // separator
    sep = gtk_hseparator_new();
    gtk_table_attach(GTK_TABLE(table), sep, 0, 6, 13, 14,
                     GTK_FILL, GTK_FILL, 0, 10);

    // dock placement and alignment
    // Top, TopRight, Right, BottomRight, Bottom, BottomLeft, Left, TopLeft
    abox = gtk_hbox_new(FALSE, 0);
    label = gtk_label_new("Dock placement and alignment");
    gtk_box_pack_start(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 0, 6, 14, 15);
    
    // dock position
    static Value dck_pos(rc.root(), "Top", "dock/position");
    label = gtk_label_new("position:");
    abox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 1, 2, 15, 16);
    const gchar *str = dck_pos.get_string();
    item = gtk_combo_box_new_text();
    const gchar *pos_name[] = {
        "Top", "TopRight", "Right", "BottomRight",
        "Bottom", "BottomLeft", "Left", "TopLeft", "Floating"
    };
    for(int i = 0; i < 9; ++i) {
        gtk_combo_box_append_text(GTK_COMBO_BOX(item), pos_name[i]);
        if(!utf8_cmp(pos_name[i], str))
            gtk_combo_box_set_active(GTK_COMBO_BOX(item), i);
    }
    g_signal_connect(G_OBJECT(item), "changed",
                     G_CALLBACK(on_change_combo), &dck_pos);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 2, 3, 15, 16);
    
    // dock direction (alignment)
    static Value dck_aln(rc.root(), "Vertical", "dock/direction");
    label = gtk_label_new("alignment:");
    abox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 1, 2, 16, 17);
    str = dck_aln.get_string();
    item = gtk_combo_box_new_text();
    const gchar *align_name[] = {
        "Horizotal", "Vertical"
    };
    for(int i = 0; i < 2; ++i) {
        gtk_combo_box_append_text(GTK_COMBO_BOX(item), align_name[i]);
        if(!utf8_cmp(align_name[i], str))
            gtk_combo_box_set_active(GTK_COMBO_BOX(item), i);
    }
    g_signal_connect(G_OBJECT(item), "changed",
                     G_CALLBACK(on_change_combo), &dck_aln);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 2, 3, 16, 17);
    
    // dock floatingX
    label = gtk_label_new("floating X:");
    abox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 4, 5, 15, 16);
    
    // dock floatingY
    label = gtk_label_new("floating Y:");
    abox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 4, 5, 16, 17);

    // dock stacking (dock layer)
    static Value dck_lay(rc.root(), "Top", "dock/stacking");
    label = gtk_label_new("desktop layer:");
    abox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 1, 2, 17, 18);
    str = dck_lay.get_string();
    item = gtk_combo_box_new_text();
    const gchar *stack_name[] = {
        "Top", "Normal", "Bottom"
    };
    for(int i = 0; i < 3; ++i) {
        gtk_combo_box_append_text(GTK_COMBO_BOX(item), stack_name[i]);
        if(!utf8_cmp(stack_name[i], str))
            gtk_combo_box_set_active(GTK_COMBO_BOX(item), i);
    }
    g_signal_connect(G_OBJECT(item), "changed",
                     G_CALLBACK(on_change_combo), &dck_lay);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 2, 3, 17, 18);
    
    // dock moveButton
    static Value dck_mvb(rc.root(), "A-Left", "dock/moveButton");
    label = gtk_label_new("move key:");
    abox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 1, 2, 18, 19);
    item = gtk_entry_new_with_max_length(10);
    gtk_entry_set_width_chars(GTK_ENTRY(item), 10);
    gtk_entry_set_alignment(GTK_ENTRY(item), 1);
    gtk_entry_set_text(GTK_ENTRY(item), dck_mvb.get_string());
    g_signal_connect(G_OBJECT(item), "key-release-event",
                     G_CALLBACK(on_change_string), &dck_mvb);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 2, 3, 18, 19);

    // dock autoHide
    static Value dck_ahd(rc.root(), false, "yes", "no", "dock/autoHide");
    item = gtk_check_button_new_with_label("auto hide");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item),
                                 dck_ahd.get_boolean());
    g_signal_connect(G_OBJECT(item), "toggled",
                     G_CALLBACK(on_change_boolean), &dck_ahd);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 4, 6, 17, 18);
    
    // dock hideDelay
    static Value dck_del(rc.root(), 300, 0, 10000,
                         "dock/hideDelay");
    label = gtk_label_new("hide delay (ms):");
    abox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(abox), label, FALSE, FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), abox, 4, 5, 18, 19);
    adj = (GtkAdjustment*) gtk_adjustment_new(
            dck_del.get_number(), dck_del.min(), dck_del.max(), 1, 5, 0);
    item = gtk_spin_button_new(adj, 0, 0);
    g_signal_connect(item, "value-changed",
                     G_CALLBACK(on_change_number), &dck_del);
    gtk_table_attach_defaults(GTK_TABLE(table), item, 5, 6, 18, 19);

    
    gtk_widget_show_all(table);
     
    return res;
}

