// fbinds.cpp

#include "fbinds.h"
#include "bind.h"

namespace Binds {
    // tree store item IDs
    enum {
        TREE_VALA = 0,
        TREE_VALB,
        TREE_PARENT,
        TREE_ITEM,
        TREE_TYPE,
        TREE_NOTE,
        TREE_SIZE
    };
    
    // tree store item type IDs
    enum {
        ITEM_TOP = 0,
        ITEM_CONTEXT,
        ITEM_BINDK,
        ITEM_BINDM,
        ITEM_ACTION,
        ITEM_VALUES,
        ITEM_VALUEN,
        ITEM_VALUEB,
        ITEM_NEWA,
        ITEM_NEWBK,
        ITEM_NEWBM,
        ITEM_SIZE
    };
    // control widget IDs
    enum {
        CTRL_STRING,
        CTRL_NUMBER,
        CTRL_BOOLEAN,
        CTRL_KEYA,
        CTRL_ACTION,
        CTRL_SIZE
    };

    // control widget show-hide table
    bool ctrl_show[ITEM_SIZE][CTRL_SIZE] = {
      // STRING NUMBER BOOLEAN KEYA   ACTION
        {false, false, false,  false, false}, // ITEM_TOP
        {false, false, false,  false, false}, // ITEM_CONTEXT
        {true,  false, false,  false, false}, // ITEM_BINDK
        {true,  false, false,  true,  false}, // ITEM_BINDM
        {false, false, false,  false, true }, // ITEM_ACTION
        {true,  false, false,  false, false}, // ITEM_VALUES
        {false, true,  false,  false, false}, // ITEM_VALUEN
        {false, false, true,   false, false}, // ITEM_VALUEB
        {false, false, false,  false, true }, // ITEM_NEWA
        {true,  false, false,  false, false}, // ITEM_NEWBK
        {true,  false, false,  true,  false}  // ITEM_NEWBM
    };

    // control widgets
    GtkWidget *controls[CTRL_SIZE];
    // help note buffer
    GtkTextBuffer *helpnote;
    // change button
    GtkWidget *change;

    void add_to_tree(Value        *value,
                     GtkTreeStore *store,
                     GtkTreeIter  *iter,
                     bool          place_under,
                     gpointer      parent,
                     const gchar  *val_a,
                     const gchar  *val_b,
                     const gchar  *note);
    void add_to_tree(Action       *action,
                     GtkTreeStore *store,
                     GtkTreeIter  *iter,
                     bool          place_under,
                     gpointer      parent,
                     const gchar  *val,
                     const gchar  *note);
    void add_to_tree(Bind         *bind,
                     GtkTreeStore *store,
                     GtkTreeIter  *iter,
                     bool          place_under,
                     gpointer      parent,
                     int           type,
                     const gchar  *val_a,
                     const gchar  *val_b,
                     const gchar  *note);
    GtkTreeStore *binds_tree(Rc &rc);
    void set_controls(int selected_type,
                      const gpointer item,
                      const gpointer parent,
                      const gchar *note);
    gboolean on_select(GtkTreeSelection *selection,
                       gpointer data);
    gboolean on_change(GtkButton *button,
                       gpointer data);
    gboolean on_action_select(GtkComboBox *action_combo,
                              gpointer tree_selection);
};

void Binds::add_to_tree(Value        *value,
                        GtkTreeStore *store,
                        GtkTreeIter  *iter,
                        bool          place_under,
                        gpointer      parent,
                        const gchar  *val_a,
                        const gchar  *val_b,
                        const gchar  *note)
{
    if(!val_a) val_a = "";
    if(!val_b) val_b = "";
    if(!note) note = "";
    GtkTreeIter it;
    if(place_under)
        gtk_tree_store_append(store, &it, iter);
    else
        gtk_tree_store_insert_before(store, &it, 0, iter);
    gtk_tree_store_set(store,      &it,
                       TREE_VALA,   val_a,
                       TREE_VALB,   val_b,
                       TREE_PARENT, parent,
                       TREE_ITEM,   value,
                       TREE_TYPE,   
                       ( (value->type() == Value::STRING) ?
                         ITEM_VALUES : 
                         ( (value->type() == Value::NUMBER) ?
                           ITEM_VALUEN : ITEM_VALUEB
                         )
                       ),
                       TREE_NOTE,   note, -1);
}

void Binds::add_to_tree(Action       *action,
                        GtkTreeStore *store,
                        GtkTreeIter  *iter,
                        bool          place_under,
                        gpointer      parent,
                        const gchar  *val,
                        const gchar  *note)
{
    if(!val) val = "";
    if(!note) note = "";
    GtkTreeIter it;
    if(place_under)
        gtk_tree_store_append(store, &it, iter);
    else
        gtk_tree_store_insert_before(store, &it, 0, iter);
    gtk_tree_store_set(store,      &it,
                       TREE_VALA,   val,
                       TREE_VALB,   0,
                       TREE_PARENT, parent,
                       TREE_ITEM,   action,
                       TREE_TYPE,   ITEM_ACTION,
                       TREE_NOTE,   note, -1);
    int act_num = action->get_pattern();
    for(int i = 0; i < 2; ++i)
        if(Value *val = action->param(i))
            add_to_tree(val, store, &it, true, action,
                        Action::patterns_param_name(act_num, i),
                        val->get_string(),
                        Action::patterns_param_note(act_num, i));
}

void Binds::add_to_tree(Bind         *bind,
                        GtkTreeStore *store,
                        GtkTreeIter  *iter,
                        bool          place_under,
                        gpointer      parent,
                        int           type,
                        const gchar  *val_a,
                        const gchar  *val_b,
                        const gchar  *note)
{
    if(!val_a) val_a = "";
    if(!val_b) val_b = "";
    if(!note) note = "";
    GtkTreeIter it;
    if(place_under) gtk_tree_store_append(store, &it, iter);
    else gtk_tree_store_insert_before(store, &it, 0, iter);
    gtk_tree_store_set(store,      &it,
                       TREE_VALA,   val_a,
                       TREE_VALB,   val_b,
                       TREE_PARENT, parent,
                       TREE_ITEM,   bind,
                       TREE_TYPE,   type,
                       TREE_NOTE,   note, -1);
    for(Action *a = bind->first_action(); a; a = bind->next_action())
        add_to_tree(a, store, &it, true, bind, a->name(),
                    Action::patterns_note(a->get_pattern()));
    for(Bind *b = bind->first_bind(); b; b = bind->next_bind())
        add_to_tree(b, store, &it, true, bind, 
                ((b->type() == Bind::KEYBOARD) ? ITEM_BINDK : ITEM_BINDM),
                b->get_key(),
                ((b->type() == Bind::MOUSE) ? 
                 Bind::key_actions_name(b->get_key_action()) : 0),
                Bind::bind_types_note(b->type()));
    if(type == ITEM_BINDK || type == ITEM_BINDM) {
        GtkTreeIter sub;
        gtk_tree_store_append(store, &sub, &it);
        gtk_tree_store_set(store,      &sub,
                           TREE_VALA,   "<new action>",
                           TREE_VALB,   0,
                           TREE_PARENT, bind,
                           TREE_ITEM,   0,
                           TREE_TYPE,   ITEM_NEWA,
                           TREE_NOTE,   "add new action", -1);
    }
    if(bind->type() == Bind::KEYBOARD || type == ITEM_CONTEXT) {
        GtkTreeIter sub;
        gtk_tree_store_append(store, &sub, &it);
        gtk_tree_store_set(store,      &sub,
                           TREE_VALA,   "[new bind]",
                           TREE_VALB,   0,
                           TREE_PARENT, bind,
                           TREE_ITEM,   0,
                           TREE_TYPE,   ((bind->type() == Bind::KEYBOARD) ? 
                                         ITEM_NEWBK : ITEM_NEWBM),
                           TREE_NOTE,   "add new bind", -1);
    }
}


GtkTreeStore *Binds::binds_tree(Rc &rc)
{
    // keyboard and mouse bind descriptions
    static const gchar *note[2] = {
        "key bindings are used to bind actions to key combinations; whenever "\
        "the combination is pressed, the actions bound to it will be executed",
        "mouse bindings are used to bind actions to mouse interactions; "\
        "these control most of Openbox's behavior, from what happens when "\
        "you click the Close button in a window's titlebar to how you drag "\
        "windows to move them around the screen", 
    };
    
    // mouse bind contexts descriptions
    static const gchar *context_note[16] = {
        "frame includes the entirety of all windows; this includes "\
        "on their decorations and in their client area; when a button "\
        "is bound in this context, the clicks will not be passed "\
        "through to the application; use with care",
        "titlebar displayed on the top of each window, with the "\
        "title, and some buttons in it",
        "handle is the decorations on the bottom of each window",
        "the bottom-left corner of the window's handle (the mouse "\
        "cursor changes while over this); generally this is only used "\
        "for resizing windows",
        "the bottom-right corner of the window's handle (the mouse "\
        "cursor changes while over this); generally this is only used "\
        "for resizing windows",
        "the top-left corner of the window's titlebar (the mouse "\
        "cursor changes while over this); generally this is only used "\
        "for resizing windows",
        "the top-right corner of the window's titlebar (the mouse "\
        "cursor changes while over this); generally this is only used "\
        "for resizing windows",
        "the client area of all windows; when a button is bound in "\
        "this context, the clicks will be passed through to the "\
        "application; because of how these are handled, press "\
        "interactions are the only useful interactions for this "\
        "context",
        "the window's icon in all window's titlebars",
        "the all-desktops (omnipresent) button in all window's "\
        "titlebars; this button is not visible in the default "\
        "settings; it is called the all-desktop button because its "\
        "default bindings toggle omnipresence on the window",
        "the shade button in all window's titlebars; this button is "\
        "not visible in the default settings; it is called the shade "\
        "button because its default bindings toggle shading on the "\
        "window",
        "the iconify button in all window's titlebars; it is called "\
        "the iconify button because its default bindings iconify the "\
        "window",
        "the maximize button in all window's titlebars; it is called "\
        "the maximize button because its default bindings maximize "\
        "the window",
        "the close button in all window's titlebars; it is called the "\
        "close button because its default bindings close the window",
        "the desktop or background; this works even when you are "\
        "running a file manager such as Nautilus",
        "bindings under this context are available while moving or "\
        "resizing a window"
    };

    static Bind keyboard(rc.root(), "keyboard", 0, 0, Bind::KEYBOARD);
    static const gchar *context[16] = {
        "Frame", 
        "Titlebar",
        "Handle",
        "BLCorner",
        "BRCorner",
        "TLCorner",
        "TRCorner",
        "Client",
        "Icon",
        "AllDesktops",
        "Shade",
        "Iconify",
        "Maximize",
        "Close",
        "Desktop",
        "MoveResize"
    };
    static Bind mouse[16] = {
        Bind(rc.root(), "mouse/context", "name", context[0], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[1], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[2], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[3], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[4], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[5], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[6], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[7], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[8], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[9], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[10], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[11], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[12], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[13], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[14], Bind::MOUSE),
        Bind(rc.root(), "mouse/context", "name", context[15], Bind::MOUSE)
    };
    GtkTreeStore *store = gtk_tree_store_new(TREE_SIZE,
            G_TYPE_STRING,  // value A
            G_TYPE_STRING,  // value B
            G_TYPE_POINTER, // parent
            G_TYPE_POINTER, // item it self
            G_TYPE_INT,     // type
            G_TYPE_STRING);// note
    add_to_tree(&keyboard, store, 0, true, 0, ITEM_TOP,
                "Keyboard", 0, note[0]);
    GtkTreeIter iter;
    gtk_tree_store_append(store, &iter, 0);
    gtk_tree_store_set(store,      &iter,
                       TREE_VALA,   "Mouse",
                       TREE_VALB,   0,
                       TREE_PARENT, 0,
                       TREE_ITEM,   0,
                       TREE_TYPE,   ITEM_TOP,
                       TREE_NOTE,   note[1], -1);
    for(int i = 0; i < 16; ++i)
        add_to_tree(&mouse[i], store, &iter, true, 0, ITEM_CONTEXT,
                    context[i], 0, context_note[i]);
    return store;
}

void Binds::set_controls(int selected_type,
                         const gpointer item,
                         const gpointer parent,
                         const gchar *note) {
    gtk_text_buffer_set_text(helpnote, note, -1);
    for(int ctrl = 0; ctrl < CTRL_SIZE; ++ctrl)
        if(ctrl_show[selected_type][ctrl])
            gtk_widget_show(controls[ctrl]);
        else
            gtk_widget_hide(controls[ctrl]);
    Bind *bind;
    Action *action;
    Value *value;
    switch(selected_type) {
        case ITEM_BINDK:
        case ITEM_BINDM:
            gtk_entry_set_text(GTK_ENTRY(controls[CTRL_STRING]),
                               ((Bind *)item)->get_key());
            if(selected_type == ITEM_BINDM)
                gtk_combo_box_set_active(GTK_COMBO_BOX(controls[CTRL_KEYA]),
                                         ((Bind *)item)->get_key_action());
            break;
        case ITEM_ACTION:
            gtk_combo_box_set_active(GTK_COMBO_BOX(controls[CTRL_ACTION]),
                                     ((Action *)item)->get_pattern());
            break;
        case ITEM_VALUES:
            gtk_entry_set_text(GTK_ENTRY(controls[CTRL_STRING]),
                               ( ((Value *)item)->get_string() ?
                                 ((Value *)item)->get_string() : "") );
            break;
        case ITEM_VALUEN:
            gtk_spin_button_set_value(GTK_SPIN_BUTTON(controls[CTRL_NUMBER]),
                                      ((Value *)item)->get_number());
            break;
        case ITEM_VALUEB:
            gtk_combo_box_set_active(GTK_COMBO_BOX(controls[CTRL_BOOLEAN]),
                                     (((Value *)item)->get_boolean() ? 0 : 1) );
            break;
        case ITEM_NEWA:
            gtk_combo_box_set_active(GTK_COMBO_BOX(controls[CTRL_ACTION]), 0);
            break;
        case ITEM_NEWBK:
            break;
        case ITEM_NEWBM:
            gtk_combo_box_set_active(GTK_COMBO_BOX(controls[CTRL_KEYA]), 0);
            break;
    }
        
    gtk_widget_set_sensitive(change, (selected_type > 1));
}

gboolean Binds::on_select(GtkTreeSelection *selection,
                          gpointer data) {
    GtkTreeModel *model;
    GtkTreeIter iter;
    if(gtk_tree_selection_get_selected(selection, &model, &iter)) {
        gchar *note;
        int type;
        gpointer item;
        gpointer parent;
        gtk_tree_model_get(model,      &iter,
                           TREE_ITEM,  &item,
                           TREE_PARENT,&parent,
                           TREE_TYPE,  &type,
                           TREE_NOTE,  &note, -1);
        set_controls(type, item, parent, note);
        if(note) g_free(note);
    }
    return true;
}

gboolean Binds::on_change(GtkButton *button,
                          gpointer data) {
    GtkTreeSelection *selection = GTK_TREE_SELECTION (data);
    GtkTreeModel *model;
    GtkTreeIter iter;
    if(gtk_tree_selection_get_selected(selection, &model, &iter)) {
        int type;
        gpointer item;
        gpointer parent;
        gtk_tree_model_get(model,       &iter,
                           TREE_ITEM,   &item,
                           TREE_PARENT, &parent,
                           TREE_TYPE,   &type, -1);
        Value *value;
        Bind *bind;
        Bind *bind2;
        Action *action;
        int number;
        const gchar *string;
        bool boolean;
        switch(type) {
            case ITEM_BINDK:
            case ITEM_BINDM:
                // change bind
                bind = (Bind *)item;
                string = gtk_entry_get_text(GTK_ENTRY(controls[CTRL_STRING]));
                if(string && string[0]) {
                    bind->set_key(string);
                    gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
                                       TREE_VALA,
                                       bind->get_key(), -1);
                    if(type == ITEM_BINDM) {
                        bind->set_key_action(gtk_combo_box_get_active(
                                GTK_COMBO_BOX(controls[CTRL_KEYA])));
                        gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
                                           TREE_VALB, 
                                           Bind::key_actions_name(
                                               bind->get_key_action()),
                                           -1);
                    }
                }
                else {
                    // delete bind
                    bind2 = (Bind *)parent;
                    bind2->del(bind);
                    gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
                }
                break;
            case ITEM_ACTION:
                bind = (Bind *)parent;
                action = (Action *)item;
                number = gtk_combo_box_get_active(
                         GTK_COMBO_BOX(controls[CTRL_ACTION]));
                if(number && number != action->get_pattern()) {
                    // change action
                    action->set_pattern(number);
                    gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
                                       TREE_VALA,
                                       Action::patterns_name(
                                           action->get_pattern()),
                                       -1);
                    GtkTreeIter sub;
                    while(gtk_tree_model_iter_children(model, &sub, &iter))
                        gtk_tree_store_remove(GTK_TREE_STORE(model), &sub);
                    for(int p = 0; p < 2; ++p) {
                        Value *param = action->param(p);
                        if(param) {
                            add_to_tree(param, GTK_TREE_STORE(model), &iter,
                                        true, action,
                                        Action::patterns_param_name(number, p),
                                        param->get_string(),
                                        Action::patterns_param_note(number, p));
                        }
                    }
                }
                else if(!number) {
                    // delete action
                    bind->del(action);
                    gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
                }
                break;
            case ITEM_VALUES:
                // change string value
                value = (Value *)item;
                string = gtk_entry_get_text(GTK_ENTRY(controls[CTRL_STRING]));
                if(utf8_cmp(value->get_string(), string)) {
                    value->set_string(string);
                    gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
                                       TREE_VALB, value->get_string(), -1);
                }
                break;
            case ITEM_VALUEN:
                // change string value
                value = (Value *)item;
                number = gtk_spin_button_get_value_as_int(
                         GTK_SPIN_BUTTON(controls[CTRL_NUMBER]));
                if(number != value->get_number()) {
                    value->set_number(number);
                    gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
                                       TREE_VALB, value->get_string(), -1);
                }
                break;
            case ITEM_VALUEB:
                // change boolean value
                value = (Value *)item;
                boolean = (gtk_combo_box_get_active(
                            GTK_COMBO_BOX(controls[CTRL_BOOLEAN])) == 0);
                if(boolean != value->get_boolean()) {
                    value->set_boolean(boolean);
                    gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
                                       TREE_VALB, value->get_string(), -1);
                }
                break;
            case ITEM_NEWA:
                // add new action
                bind = (Bind *)parent;
                number = gtk_combo_box_get_active(
                         GTK_COMBO_BOX(controls[CTRL_ACTION]));
                if(number && (action = bind->add(number)))
                    add_to_tree(action, GTK_TREE_STORE(model), &iter,
                                false, bind, action->name(),
                                Action::patterns_note(number));
                break;
            case ITEM_NEWBK:
            case ITEM_NEWBM:
                // add new bind
                bind = (Bind *)parent;
                string = gtk_entry_get_text(GTK_ENTRY(controls[CTRL_STRING]));
                if(type == ITEM_NEWBM)
                    number = gtk_combo_box_get_active(
                             GTK_COMBO_BOX(controls[CTRL_KEYA]));
                if(string && string[0] && 
                   (bind2 = 
                    ( type == ITEM_NEWBM ? bind->add(string, number) :
                                           bind->add(string)
                    ))) {
                    add_to_tree(bind2, GTK_TREE_STORE(model),
                            &iter, false, bind, 
                            ((bind2->type() == Bind::KEYBOARD) ?
                             ITEM_BINDK : ITEM_BINDM),
                            bind2->get_key(),
                            ((bind2->type() == Bind::MOUSE) ?
                             Bind::key_actions_name(bind2->get_key_action()) :
                             0),
                            Bind::bind_types_note(bind2->type()));
                }
                break;
        }
    }
    return true;
}

gboolean Binds::on_action_select(GtkComboBox *action_combo,
                                 gpointer tree_selection) {
    GtkTreeSelection *selection = GTK_TREE_SELECTION(tree_selection);
    GtkTreeModel *model;
    GtkTreeIter iter;
    int type = ITEM_TOP;
    if(gtk_tree_selection_get_selected(selection, &model, &iter))
        gtk_tree_model_get(model,       &iter,
                           TREE_TYPE,   &type, -1);
    int num = gtk_combo_box_get_active(action_combo);
    gtk_text_buffer_set_text(helpnote,
                             (num ? Action::patterns_note(num) :
                                    ( (type == ITEM_ACTION) ? "delete action" :
                                                              "select to add" )
                             ), -1);
    return true;
}

GtkWidget *Binds::face(Rc &rc) {
    // binds tree
    GtkTreeStore *store = binds_tree(rc);
    GtkWidget *view =
        gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
    GtkTreeSelection *selection =
        gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
    g_signal_connect(G_OBJECT(selection), "changed",
                     G_CALLBACK (on_select), 0);
    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
    GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
    GtkTreeViewColumn *col = gtk_tree_view_column_new();
    GtkCellRenderer *rend = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, rend, false);
    gtk_tree_view_column_set_attributes(col, rend, "text", 0, 0);
    rend = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, rend, false);
    gtk_tree_view_column_set_attributes(col, rend, "text", 1, 0);
    gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
    // put tree in scrolled window
    GtkWidget* scr = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scr),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_ALWAYS);
    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scr), view);
    gtk_widget_show(view);
    gtk_widget_show(scr);
    gtk_widget_set_size_request(scr, 300, 250);
    
    controls[CTRL_STRING] = gtk_entry_new();
    controls[CTRL_NUMBER] = gtk_spin_button_new_with_range(-100, 100, 1); 
    controls[CTRL_BOOLEAN] = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX (controls[CTRL_BOOLEAN]), "yes");
    gtk_combo_box_append_text(GTK_COMBO_BOX (controls[CTRL_BOOLEAN]), "no");
    controls[CTRL_KEYA] = gtk_combo_box_new_text();
    for(int i = 0; Bind::key_actions_name(i); ++i)
        gtk_combo_box_append_text(GTK_COMBO_BOX (controls[CTRL_KEYA]),
                                  Bind::key_actions_name(i));
    controls[CTRL_ACTION] = gtk_combo_box_new_text();
    g_signal_connect(G_OBJECT(controls[CTRL_ACTION]), "changed",
                     G_CALLBACK (on_action_select), selection);
    gtk_combo_box_set_wrap_width(GTK_COMBO_BOX (controls[CTRL_ACTION]), 4);
    gtk_combo_box_append_text(GTK_COMBO_BOX (controls[CTRL_ACTION]),
                              "---");
    for(int i = 1; Action::patterns_name(i); ++i)
        gtk_combo_box_append_text(GTK_COMBO_BOX (controls[CTRL_ACTION]),
                                  Action::patterns_name(i));
                
    // set button
    change = gtk_button_new_with_label("change");
    gtk_widget_set_size_request(change, 200, -1);
    g_signal_connect(G_OBJECT(change), "clicked",
                     G_CALLBACK (on_change), selection);

    // help note
    helpnote = gtk_text_buffer_new(0);
    GtkWidget *helpview = gtk_text_view_new_with_buffer(helpnote);
    gtk_text_view_set_editable(GTK_TEXT_VIEW (helpview), false);
    gtk_text_view_set_justification(GTK_TEXT_VIEW (helpview), GTK_JUSTIFY_LEFT);
    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW (helpview), GTK_WRAP_WORD);
    GtkWidget* helpscr = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(helpscr),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_ALWAYS);
    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(helpscr),
                                          helpview);
    gtk_widget_set_size_request(helpscr, -1, 100);

    GtkWidget *header = gtk_hbox_new(false, 5);
    GtkWidget *ctrl_box = gtk_vbox_new(false, 5);
    gtk_box_pack_start(GTK_BOX (ctrl_box), change, false, false, 0);
    gtk_box_pack_start(GTK_BOX (ctrl_box), controls[CTRL_STRING],
                       false, false, 0);
    gtk_box_pack_start(GTK_BOX (ctrl_box), controls[CTRL_NUMBER],
                       false, false, 0);
    gtk_box_pack_start(GTK_BOX (ctrl_box), controls[CTRL_BOOLEAN],
                       false, false, 0);
    gtk_box_pack_start(GTK_BOX (ctrl_box), controls[CTRL_KEYA],
                       false, false, 0);
    gtk_box_pack_start(GTK_BOX (ctrl_box), controls[CTRL_ACTION],
                       false, false, 0);
    gtk_box_pack_start(GTK_BOX (header), scr, true, true, 0);
    gtk_box_pack_start(GTK_BOX (header), ctrl_box, false, false, 0);
    GtkWidget *box = gtk_vbox_new(false, 5);
    gtk_box_pack_start_defaults(GTK_BOX (box), header);
    gtk_box_pack_start_defaults(GTK_BOX (box), helpscr);
    gtk_widget_show_all(box);
   
    set_controls(0, 0, 0, "select item in tree to change"); 
    
    // 
    return box;
}
