// fdesks.cpp

#include "fdesks.h"
#include "value.h"

namespace Desks {
    GtkListStore *desks_store(Rc &rc);
    gboolean update_number(GtkWidget *widget,
                           gpointer value);
    gboolean update_name(GtkCellRendererText *rendered,
                         const gchar *path,
                         const gchar *value,
                         gpointer data);
};

GtkListStore *Desks::desks_store(Rc &rc)
{
    xmlNodePtr first_name =
        xml_find(rc.root(), "desktops/names/name", true, "first");
    GtkListStore *store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_POINTER);
    xmlNodePtr name = first_name;
    GtkTreeIter it;
    while(name) {
        gchar *val = xml_content(name);
        if(val) {
            gtk_list_store_append(store, &it);
            gtk_list_store_set(store, &it, 0, val, 1, name, -1);
            g_free(val);
            name = xml_next(name);
        }
        else {
            xmlNodePtr del = name;
            name = xml_next(name);
            xml_remove(del);
        }
    }
    gtk_list_store_append(store, &it);
    gtk_list_store_set(store, &it, 0, "(add new)", 1, 0, -1);
    return store;
}

gboolean Desks::update_number(GtkWidget *widget,
                              gpointer   value)
{
    int number = ((Value *)value)->get_number();
    int setnum = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
    if(setnum != number)
        ((Value *)value)->set_number(setnum);
    return true;
}

gboolean Desks::update_name(GtkCellRendererText *rendered,
                            const gchar *path,
                            const gchar *value,
                            gpointer data)
{
    GtkListStore *store = GTK_LIST_STORE(data);
    GtkTreeIter iter;
    if(gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(store),
                                           &iter, path)) {
        gchar *old_val;
        gpointer node;
        gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
                           0, &old_val, 1, &node, -1);
        if(value && value[0]) {
            // given new non empty value, set new value
            if(node) {
                // selected existing desktop name
                gtk_list_store_set(store, &iter, 0, value, -1);
                xml_content((xmlNodePtr)(node), value);
            } else {
                // selected <add new> item, add new desktop name
                // find last desktop name (previous iter)
                GtkTreePath *p = gtk_tree_model_get_path(GTK_TREE_MODEL(store),
                                                         &iter);
                if(gtk_tree_path_prev(p) &&
                   gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, p)) {
                    gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
                                       1, &node, -1);
                    xmlNodePtr next_node = xml_next((xmlNodePtr)node, true);
                    xml_content(next_node, value);
                    GtkTreeIter next_iter;
                    gtk_list_store_insert_after(store, &next_iter, &iter);
                    gtk_list_store_set(store, &next_iter,
                                       0, value,
                                       1, next_node, -1);
                }
                gtk_tree_path_free(p);
            }
        }
        else if(utf8_cmp(path, "0") && node) {
            // given empty value, try to delete desktop name
            GtkTreePath *p = gtk_tree_model_get_path(GTK_TREE_MODEL(store),
                                                     &iter);
            if(gtk_tree_path_prev(p)) {
                gtk_list_store_remove(store, &iter);
                xml_remove((xmlNodePtr)(node));
            }
            gtk_tree_path_free(p);
        }
        else if(gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter) &&
                gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter)) {
            gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
            gtk_list_store_remove(store, &iter);
        }
    }
    return true;
}

GtkWidget *Desks::face(Rc &rc)
{
    static Value number(rc.root(), 4, 1, 20, "desktops/number");
    GtkListStore *store = desks_store(rc);
    GtkWidget *view =
        gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
    gtk_tree_selection_set_mode(
            gtk_tree_view_get_selection(GTK_TREE_VIEW(view)),
            GTK_SELECTION_SINGLE);
    GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
    GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes(
            "desktops names", renderer, "text", 0, 0);
    gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
    g_signal_connect(G_OBJECT(renderer), "edited",
                     G_CALLBACK(update_name), store);
    g_object_set(renderer, "editable", true, 0);
    GtkWidget* scrolled = gtk_scrolled_window_new(0, 0);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_ALWAYS);
    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled), view);
    gtk_widget_show(view);

    GtkWidget *number_box = gtk_hbox_new(false, 5);
    GtkAdjustment *adj = 
        GTK_ADJUSTMENT(gtk_adjustment_new(number.get_number(), 1, 20, 1, 5, 0));
    GtkWidget *number_spin = gtk_spin_button_new(adj, 0, 0);
    gtk_widget_show(number_spin);
    g_signal_connect(number_spin, "value-changed", 
                     G_CALLBACK(update_number), &number);
    GtkWidget *label = gtk_label_new("desktops number");
    gtk_widget_show(label);
    GtkWidget *label_box = gtk_hbox_new(false, 0);
    gtk_box_pack_start(GTK_BOX(label_box), label, false, false, 0);
    gtk_widget_show(label);
    gtk_widget_show(label_box);
    gtk_box_pack_start(GTK_BOX(number_box), number_spin, false, false, 0);
    gtk_box_pack_start(GTK_BOX(number_box), label_box, true, true, 0);

    GtkWidget *box = gtk_vbox_new(false, 5);
    gtk_box_pack_start(GTK_BOX(box), number_box, false, false, 0);
    gtk_box_pack_start(GTK_BOX(box), scrolled, true, true, 0);
    gtk_widget_show(number_box);
    gtk_widget_show(scrolled);
    return box;
}

