// fthemes.cpp

#include "fthemes.h"

#include <openbox/parse.h>

namespace Themes {
    Rc *rc_data = 0;
    gboolean on_select(GtkTreeSelection *selection,
                       gpointer value);
    gboolean on_title(GtkWidget* widget,
                      GdkEventKey *event,
                      gpointer value);
};

gboolean Themes::on_select(GtkTreeSelection *selection,
                           gpointer          value)
{
    GtkTreeIter iter;
    GtkTreeModel *model;
    if(gtk_tree_selection_get_selected(selection, &model, &iter)) {
        gchar *p;
        gtk_tree_model_get(model, &iter, 0, &p, -1);
        ((Value *) value)->set_string(p);
        g_free(p);
        rc_data->save();
        rc_data->apply();
    }
    return true;
}

gboolean Themes::on_title(GtkWidget   *widget,
                          GdkEventKey *event,
                          gpointer     value)
{
    ((Value *) value)->set_string(gtk_entry_get_text(GTK_ENTRY(widget)));
    rc_data->save();
    rc_data->apply();
    return true;
}

GtkWidget *Themes::face(Rc &rc)
{
    rc_data = &rc;
    GtkWidget *button_box = gtk_hbox_new(false, 5);
    GtkWidget *label = gtk_label_new("title layout");
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(button_box), label, false, false, 0);
    GtkWidget *layout = gtk_entry_new();
    GtkTooltips *tooltips = gtk_tooltips_new();
    gtk_tooltips_set_tip(tooltips, layout,
                         "N - system window icon\n"\
                         "D - sticky button (place to all desktops)\n"\
                         "S - shade button (roll up)\n"\
                         "L - window label\n"\
                         "I - iconify button (minimize)\n"\
                         "M - maximize button\n"\
                         "C - close button", 0);
    static Value title_string(rc.root(), "NLIMC", "theme/titleLayout");
    gtk_entry_set_text(GTK_ENTRY(layout), title_string.get_string());
    gtk_widget_show(layout);
    gtk_box_pack_start(GTK_BOX(button_box), layout, true, true, 0);
    g_signal_connect(G_OBJECT(layout), "key-release-event",
                     G_CALLBACK(on_title), &title_string);
    gtk_widget_show(button_box);
    
    
    static Value theme_name(rc.root(), "TheBear", "theme/name");
    const gchar *curr_name = theme_name.get_string();
    GSList *dirs = g_slist_append(0, g_build_filename(g_get_home_dir(), 
                                  ".themes", 0));
    for(GSList *curr = parse_xdg_data_dir_paths(); curr;
                curr = g_slist_next(curr)) 
        dirs = g_slist_append(dirs, g_build_filename((gchar*)curr->data,
                              "themes", 0));
    GSList *names = 0;
    for(GSList *curr = dirs; curr; curr = g_slist_next(curr)) {
        GDir *dir;
        const gchar *name;
        if((dir = g_dir_open((gchar*)curr->data, 0, 0))) {
            while((name = g_dir_read_name(dir))) {
                gchar *p = g_build_filename((gchar*)curr->data, name,
                                     "openbox-3", "themerc", 0);
                if(g_file_test(p, 
                                (GFileTest)(G_FILE_TEST_IS_REGULAR | 
                                G_FILE_TEST_IS_SYMLINK)))
                    names = g_slist_append(names, g_strdup(name));
                g_free(p);
            }
        }
        g_free(curr->data);
    }
    g_slist_free(dirs);
    names = g_slist_sort(names,
                         (gint (*)(const void*, const void*))g_utf8_collate);
    GtkListStore *store = gtk_list_store_new(1, G_TYPE_STRING);
    GtkWidget *view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
    GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
    g_signal_connect(G_OBJECT(sel), "changed",
                     G_CALLBACK(select), &theme_name);
    gtk_tree_selection_set_mode(
            gtk_tree_view_get_selection(GTK_TREE_VIEW(view)),
            GTK_SELECTION_SINGLE);
    GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
    GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes(
            "installed themes", renderer, "text", 0, 0);
    gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
    GtkTreeIter theme;
    for(GSList *curr = names; curr; curr = g_slist_next(curr)) {
        gtk_list_store_append(store, &theme);
        gtk_list_store_set(store, &theme, 0, (gchar *)curr->data, -1);
        if(!g_utf8_collate(curr_name, (gchar *)curr->data))
            gtk_tree_selection_select_iter(sel, &theme);
        g_free(curr->data);
    }
    g_slist_free(names);
    GtkWidget* scrolled = gtk_scrolled_window_new(0, 0);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_ALWAYS);
    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled), view);
    gtk_widget_show(view);
    gtk_widget_show(scrolled);

    GtkWidget *box = gtk_vbox_new(false, 5);
    gtk_box_pack_start(GTK_BOX(box), button_box, false, false, 0);
    gtk_box_pack_end(GTK_BOX(box), scrolled, true, true, 0);

    return box;
}

