/* main.cpp */

#include "rc.h"
#include "fbehav.h"
#include "fbinds.h"
#include "fdesks.h"
#include "fthemes.h"
#include <gtk/gtk.h>

gboolean on_press_button(GtkWidget *button,
                         gpointer   data)
{
    const gchar *label = gtk_button_get_label((GtkButton *) button);
    if(!g_utf8_collate(label, "ok")) {
        ((Rc *) data)->save();
        ((Rc *) data)->apply();
        gtk_main_quit();
        return true;
    }
    else if(!g_utf8_collate(label, "quit")) {
        ((Rc *) data)->restore();
        ((Rc *) data)->apply();
        gtk_main_quit();
        return true;
    }
    return false;
}

void face(Rc &rc)
{
    GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

    // notebook on main window
    GtkWidget *notebook = gtk_notebook_new();
    gtk_widget_show(notebook);

    // behavior notebook page
    GtkWidget *content = Behavior::face(rc);
    GtkWidget *frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
    gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
    gtk_container_add(GTK_CONTAINER(frame), content);
    GtkWidget *label = gtk_label_new("behavior");
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);
    gtk_widget_show(content);
    gtk_widget_show(label);
    gtk_widget_show(frame);

    // binds notebook page
    content = Binds::face(rc);
    frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
    gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
    gtk_container_add(GTK_CONTAINER(frame), content);
    label = gtk_label_new("bindings");
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);
    gtk_widget_show(content);
    gtk_widget_show(label);
    gtk_widget_show(frame);

    // desktops notebook page
    content = Desks::face(rc);
    frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
    gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
    gtk_container_add(GTK_CONTAINER(frame), content);
    label = gtk_label_new("desktops");
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);
    gtk_widget_show(content);
    gtk_widget_show(label);
    gtk_widget_show(frame);

    // themes
    content = Themes::face(rc);
    frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
    gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
    gtk_container_add(GTK_CONTAINER(frame), content);
    label = gtk_label_new("theme");
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);
    gtk_widget_show(content);
    gtk_widget_show(label);
    gtk_widget_show(frame);

    /*
    // menus
    content = FaceMenus::frame(data, window);
    frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
    gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
    gtk_container_add(GTK_CONTAINER(frame), content);
    label = gtk_label_new("menus");
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);
    gtk_widget_show(content);
    gtk_widget_show(label);
    gtk_widget_show(frame);
    */

    // ok and quit buttons
    GtkWidget *ok_quit_box = gtk_hbox_new(TRUE, 5);
    GtkWidget *button = gtk_button_new_with_label("quit");
    gtk_box_pack_start(GTK_BOX(ok_quit_box), button, TRUE, TRUE, 0);
    g_signal_connect(G_OBJECT(button), "clicked",
                     G_CALLBACK(on_press_button), &rc);
    GtkTooltips *tooltips = gtk_tooltips_new();
    gtk_tooltips_set_tip(tooltips, button, "do not save and quit", 0);
    gtk_widget_show(button);
    button = gtk_button_new_with_label("ok");
    gtk_box_pack_end(GTK_BOX(ok_quit_box), button, TRUE, TRUE, 0);
    g_signal_connect(G_OBJECT(button), "clicked",
                     G_CALLBACK(on_press_button), &rc);
    gtk_tooltips_set_tip(tooltips, button, "save changes and quit", 0);
    gtk_widget_show(button);
    gtk_widget_show(ok_quit_box);

    // whole window box
    GtkWidget *common_box = gtk_vbox_new(FALSE, 5);
    gtk_box_pack_start(GTK_BOX(common_box), notebook, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(common_box), ok_quit_box, FALSE, FALSE, 0);
    gtk_widget_show(common_box);

    gtk_window_set_title(GTK_WINDOW(window), "ObTuner");
    gtk_container_set_border_width (GTK_CONTAINER(window), 2);
    gtk_container_add(GTK_CONTAINER(window), common_box);
    g_signal_connect(G_OBJECT(window), "destroy",
                     G_CALLBACK (gtk_main_quit), NULL);
    g_signal_connect(G_OBJECT(window), "delete_event",
                     G_CALLBACK (gtk_main_quit), NULL);
    
    gtk_widget_show(window);
    gtk_main();
}

int main(int   argc,
         char *argv[])
{
    try {
        Rc rc;
        rc.backup();
        gtk_init(&argc, &argv);
        face(rc);
    }
    catch(...) {
        g_error("! some error\n");
    }
    return 0;
}

