// rc.cpp

#include "rc.h"

#include <openbox/parse.h>
#include <gtk/gtk.h>
#include <signal.h>

xmlDocPtr Rc::_doc = 0;
xmlNodePtr Rc::_root = 0;
gchar *Rc::_path = 0;
int Rc::_counter = 0;

Rc::Rc()
{
    if(!_counter) {
        parse_paths_startup();
        const gchar *pc = parse_xdg_config_home_path();
        parse_mkdir_path(pc, 0700);
        _path = g_build_filename(pc, "openbox", "rc.xml", 0);
        if(!parse_load_rc(&_doc, &_root)) {
            g_message("can not load rc.xml");
            throw;
        }
    }
    ++_counter;
}

Rc::~Rc()
{
    if(_counter == 1) {
        g_free(_path);
        parse_close(_doc);
        parse_paths_shutdown();
    }
    --_counter;
}

const xmlDocPtr Rc::doc()
{
    return _doc;
}

const xmlNodePtr Rc::root()
{
    return _root;
}

void Rc::backup() {
    if(_doc && _path) {
        gchar *pc = g_strjoin(0, _path, "~", 0);
        xmlSaveFormatFileEnc(pc, _doc, "UTF-8", 1);
        g_free(pc);
    }
}

void Rc::restore() {
    gchar *pc = g_strjoin(0, "cp ", _path, "~ ", _path, 0);
    system(pc);
    g_free(pc);
}

void Rc::save() {
    if(_doc && _path) 
        xmlSaveFormatFileEnc(_path, _doc, "UTF-8", 1);
}

void Rc::apply() {
    GdkAtom type;
    gint format;
    gint length;
    guint *pid;
    if(gdk_property_get(gdk_screen_get_root_window(
                            gdk_screen_get_default()),
                        gdk_atom_intern("_OPENBOX_PID", false),
                        gdk_atom_intern("CARDINAL", false),
                        0, 4, false, &type, &format, &length,
                        (guchar**)&pid)) {
        kill(*pid, SIGUSR2);
        g_free(pid);
    }
}
    
