// value.cpp

#include "value.h"

Value::Value(xmlNodePtr   root,
             int          def,
             int          min,
             int          max,
             const gchar *path) :
    _type(NUMBER),
    _true(0), _false(0),
    _min(min), _max(max)
{
    gchar buf[10];
    g_sprintf(buf, "%d", def);
    _node = xml_find(root, path, true, buf, 0, 0);
}       
    
Value::~Value()
{ 
    if(_true) g_free(_true);
    if(_false) g_free(_false);
}

const xmlNodePtr Value::node(const xmlNodePtr new_node)
{
    return (new_node ? (_node = new_node) : _node);
}

Value::Type Value::type(Value::Type new_type)
{ 
    return ((new_type == NONE) ? _type : (_type = new_type));
}

void Value::remove()
{
    if(_node)
        xml_remove(_node);
}

int Value::min() const
{
    return _min;
}

int Value::max() const
{
    return _max;
}

const gchar *Value::get_string() const
{
    static gchar *buf(0);
    if(buf)
        g_free(buf);
    buf = xml_content(_node);
    return buf;
}

int Value::get_number() const
{
    return (_node ? parse_int(_node->doc, _node) : 0);
}

bool Value::get_boolean() const
{
    if(!_node)
        return false;
    if(!_true || !_false)
        return parse_bool(_node->doc, _node);
    else
        return !utf8_cmp(get_string(), _true);
}

void Value::set_string(const gchar *new_string)
{
    xml_content(_node, new_string);
}

void Value::set_number(int new_number)
{
    if(_node) {
        gchar buf[10];
        g_sprintf(buf, "%d", new_number);
        set_string(buf);
    }
    else
        new_number = 0;
}

void Value::set_boolean(bool new_boolean)
{
    if(_node)
        set_string( new_boolean ?
                (_true ? _true : "yes") :
                (_false ? _false : "no") );
    else
        new_boolean = false;
}

