// value.h

#ifndef _value_h_included_
#define _value_h_included_

#include "xml.h"
#include <openbox/parse.h>
#include <glib/gprintf.h>

class Value
{
public:
    // node type
    enum Type { STRING, NUMBER, BOOLEAN, MENU, PATH, NONE };

private:
    Type           _type;  // node type
    gchar         *_true;  // true string
    gchar         *_false; // false string
    int            _max;   // max value
    int            _min;   // min value
    xmlNodePtr     _node;  // xml node

public:
    
    // find (create new) node by root node, path, attribute name/value
    Value(xmlNodePtr   root,
          Type         type,
          const gchar *path,
          const gchar *def) :
        _type(type),
        _true(0), _false(0),
        _min(0), _max(0),
        _node(xml_find(root, path, true, def, 0, 0))
    {}
    
    // find (create) string value node
    Value(xmlNodePtr   root,
          const gchar *def,
          const gchar *path) :
        _type(STRING),
        _true(0), _false(0),
        _min(0), _max(0),
        _node(xml_find(root, path, true, def, 0, 0))
    {}
    
    // find (create) number value node
    Value(xmlNodePtr   root,
          int          def,
          int          min,
          int          max,
          const gchar *path);
    
    // find (create) boolean value node
    Value(xmlNodePtr   root,
          bool         def,
          const gchar *true_str,
          const gchar *false_str,
          const gchar *path) :
        _type(BOOLEAN), 
        _true(g_strdup(true_str)), _false(g_strdup(false_str)),
        _min(0), _max(0),
        _node(def ? xml_find(root, path, true, _true, 0, 0) :
                    xml_find(root, path, true, _false, 0, 0))
    {}
    
    ~Value();

    // get and set xml node
    const xmlNodePtr node(const xmlNodePtr new_node = 0);

    // get and set type
    Type type(Type new_type = NONE);

    // unlink node from xml structure and free memory
    void remove();

    // get min and max number value
    int min() const;
    int max() const;

    // get and set STRING, NUMBER and BOOLEAN values 
    const gchar *get_string() const;
    int get_number() const;
    bool get_boolean() const;
    void set_string(const gchar *new_string);
    void set_number(int new_number);
    void set_boolean(bool new_boolean);
    
};

#endif
